// ************************************************************************** //
//
//  BornAgain: simulate and fit scattering at grazing incidence
//
//! @file      Core/HardParticle/ProfileBar.h
//! @brief     Defines class ProfileBar.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
// ************************************************************************** //

#ifndef PROFILEBAR_H
#define PROFILEBAR_H

#include "IFormFactorBorn.h"
#include "Integrator.h"

//! Base class for form factors with a cosine ripple profile in the yz plane.

class BA_CORE_API_ ProfileBar : public IFormFactorBorn
{
public:
    ProfileBar(double length, double width, double height);

    double getLength() const { return m_length; }
    double getHeight() const { return m_height; }
    double getWidth() const { return m_width; }

    double radialExtension() const override final;

    complex_t evaluate_for_q(cvector_t q) const override final
    {
        return factor_x(q.x()) * factor_yz(q.y(), q.z());
    }

protected:
    void onChange() override final;
    virtual complex_t factor_x(complex_t qx) const = 0;

    double m_length;
    double m_width;
    double m_height;

private:
    complex_t factor_yz(complex_t qy, complex_t qz) const;
    bool check_initialization() const;
    mutable ComplexIntegrator m_integrator;
};

#endif // PROFILEBAR_H
